# @summary Installs Sensu plugins
#
# Installs the Sensu community script and plugins
# which can be used as monitoring checks
#
# @param type Plugin source
#   Valid values: file, directory, package, url
#
# @param install_path The path to install the plugin
#
# @param purge When using a directory source, purge setting
#
# @param recurse When using a directory source, recurse setting
#
# @param force When using a directory source, force setting
#
# @param pkg_version When using package source, version to install
#
# @param pkg_provider When using package to install plugins, provider to use.
#   Valid values: sensuclassic_gem, apt, aptitude, yum
#
# @param pkg_checksum The packake's MD5 checksum.
#   Valid values: Any valid MD5 string of the wanted package
#
# @param nocheckcertificate When using url source, disable certificate checking for HTTPS
#
# @param gem_install_options Optional configuration to use for the installation of the
#   sensu plugin gem with sensuclassic_gem provider.
#   See: https://docs.puppetlabs.com/references/latest/type.html#package-attribute-install_options
#   Example value: [{ '-p' => 'http://user:pass@myproxy.company.org:8080' }]
#
# @param pkg_proxy_host
#   The proxy host used to download the plugin when `type` is `url`
#
# @param pkg_proxy_port
#   The proxy port used to download the plugin when `type` is `url`
#
define sensuclassic::plugin (
  Enum['file','url','package','directory'] $type = 'file',
  Stdlib::Absolutepath $install_path = $::osfamily ? {
    'windows' => 'C:/opt/sensu/plugins',
    default   => '/etc/sensu/plugins',
  },
  Boolean $purge = true,
  Boolean $recurse = true,
  Boolean $force = true,
  Pattern[/^absent$/,/^installed$/,/^latest$/,/^present$/,/^[\d\.\-]+$/] $pkg_version = 'latest',
  Optional[String] $pkg_provider = $sensuclassic::sensu_plugin_provider,
  Optional[String] $pkg_checksum = undef,
  Boolean $nocheckcertificate  = false,
  Any $gem_install_options = $sensuclassic::gem_install_options,
  Optional[String] $pkg_proxy_host = $sensuclassic::package_proxy_host,
  Optional[Stdlib::Port] $pkg_proxy_port = $sensuclassic::package_proxy_port,
) {

  File {
    owner => 'sensu',
    group => 'sensu',
  }

  Sensuclassic::Plugin[$name]
  ~> Service['sensu-client']

  # (#463) All plugins must come before all checks. Collections are not used to
  # avoid realizing any resources.
  Sensuclassic::Plugin[$name]
  -> Anchor['plugins_before_checks']

  case $type {
    'file': {
      $filename = basename($name)

      sensuclassic::plugins_dir { "${name}-${install_path}":
        path    => $install_path,
        purge   => $purge,
        recurse => $recurse,
        force   => $force,
      }

      file { "${install_path}/${filename}":
        ensure  => file,
        mode    => '0555',
        source  => $name,
        require => File[$install_path],
      }
    }
    'url': {
      $filename = basename($name)

      sensuclassic::plugins_dir { "${name}-${install_path}":
        path    => $install_path,
        purge   => $purge,
        recurse => $recurse,
        force   => $force,
      }

      remote_file { $name:
        ensure     => present,
        path       => "${install_path}/${filename}",
        source     => $name,
        checksum   => $pkg_checksum,
        proxy_host => $pkg_proxy_host,
        proxy_port => $pkg_proxy_port,
        require    => File[$install_path],
      }

      file { "${install_path}/${filename}":
        ensure  => file,
        mode    => '0555',
        require => [
          File[$install_path],
          Remote_file[$name],
        ],
      }
    }
    'directory': {
      file { "${install_path}_for_plugin_${name}":
        ensure  => 'directory',
        path    => $install_path,
        mode    => '0555',
        source  => $name,
        recurse => $recurse,
        purge   => $purge,
        force   => $force,
        require => Package[$sensuclassic::package::pkg_title],
      }
    }
    'package': {
      $gem_install_options_real = $pkg_provider ? {
        'gem'              => $gem_install_options,
        'sensuclassic_gem' => $gem_install_options,
        default            => undef,
      }

      package { $name:
        ensure          => $pkg_version,
        provider        => $pkg_provider,
        install_options => $gem_install_options_real,
      }
    }
    default:      {
      fail('Unsupported sensuclassic::plugin install type')
    }
  }
}
